#ifndef STATEACCEPTOR_HH_
#define STATEACCEPTOR_HH_


#include "ell/State.hh"
#include "ell/StateCollector.hh"

namespace ell {
	/*!
	 * This abstract class is used to check if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class StateAcceptor 
	{
	public:	
		
		virtual ~StateAcceptor(){};
		
		/*!
		 * Returns true, if State succ is a valid successor to
		 * the states colleted by StateCollector sc.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const = 0;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium that accepts all states as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_All : public StateAcceptor
	{
	public:
		
		virtual ~SA_All() {};
		
		/*!
		 * Always returns true.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return returns always true
		 */
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_E_Eq : public StateAcceptor
	{
	public:
		
		virtual ~SA_E_Eq() {}
		
		/*!
		 * Tests for equal energy of State succ and last accepted State in
		 * StateCollector sc.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */		
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_E_Gr : public StateAcceptor
	{
	public:
		
		virtual ~SA_E_Gr() {}
		
		/*!
		 * Tests if energy of State succ is greater than energy of last accepted
		 * State of StateCollector sc.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */		
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_E_GrEq : public StateAcceptor
	{
	public:
		
		virtual ~SA_E_GrEq () {};
		
		/*!
		 * Tests if energy of State succ is greater or equal to energy of last 
		 * accepted State of StateCollector sc.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */	
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_E_Le : public StateAcceptor
	{
	public:
		
		virtual ~SA_E_Le() {}
		
		/*!
		 * Tests if energy of State succ is less than energy of last 
		 * accepted State of StateCollector sc.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */	
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_E_LeEq : public StateAcceptor
	{
	public:
		
		virtual ~SA_E_LeEq() {}
		
		/*!
		 * Tests if energy of State succ is less or equal to energy of last 
		 * accepted State of StateCollector sc.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */	
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements a criterium to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * To be accepted the new state has to have a lesser energy or if the energy
	 * is equal the compressed representation has to be lexicographically 
	 * smaller.
	 * 
	 * @author Martin Mann
	 */	
	class SA_EC_Le : public StateAcceptor
	{
	public:
		
		virtual ~SA_EC_Le() {}
		
		/*!
		 * Returns true, if energy of State succ is less than energy of
		 * last accepted State in StateCollector sc or if their energies are
		 * equal and the succ State has a lexicographically smaller compressed
		 * representation.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */	
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
	/*!
	 * This subclass of StateAcceptor
	 * implements the Metropolis criterion to determine if a given State is
	 * valid to be accepted as successor in a walk.
	 * 
	 * @author Daniel Maticzka
	 */	
	class SA_Metropolis : public StateAcceptor {
	protected:
		const double beta;

	public:
		SA_Metropolis(const double _beta);

		virtual ~SA_Metropolis() {}
		
		/*!
		 * Checks if State succ is valid successor to last accepted State
		 * from StateCollector sc, using the metropolis criterion.
		 * @param sc the StateCollector to test against
		 * @param succ the successor State to test
		 * @return true if valid successor
		 */
		virtual bool accept(
				const StateCollector* const sc, 
				const State& succ) const;
	};
	
} // namespace ell

#endif /*STATEACCEPTOR_HH_*/
