#ifndef S_SG_ISING_HH_
#define S_SG_ISING_HH_

#include "ell/State.hh"
#include "ell/IterableNeighbors.hh"
#include <biu/assertbiu.hh>
#include <biu/Alphabet.hh>

namespace ell
{
	// A compressed representation of an S_SG_Ising object.
	class CS_SG_Ising;

	/*!
	 * Representing a connected ring or spins. 
	 * 
	 * Spins are neighbored iif : i+1 == j or (i==0 and j==(spinnumber-1))
	 * 
	 */
	class S_SG_Ising : public State, public IterableNeighbors 
	{
		friend class CS_SG_Ising; // compressed state is friend class
		
	public:
		  //! spin representation
		typedef biu::Alphabet::AlphElem SpinType;
		  //! type of the spin array representation
		typedef biu::Alphabet::Sequence SpinArray;
		
		static const SpinType SPIN_UP;
		static const SpinType SPIN_DOWN;

	protected:

		  //! the spin alphabet class for compression usage
		static const biu::Alphabet spinAlph;

		  //! the spin array representing the +-1 spin
		SpinArray spin;	
		
	public:
		
		/*! 
		 * Constructs a spin array of the given size and initializes with 
		 * SPIN_UP for all spins or a random distribution.
		 * @param size the size of the spin array
		 * @param random if true a random initialization is done
		 */ 
		S_SG_Ising(size_t size, bool random = true);

		 //! Constructs a spin array from the given spin string
		 //! @param spinStr the +- spin string 
		S_SG_Ising(const std::string& spinStr);
		
		S_SG_Ising(const S_SG_Ising& s2);
		
		virtual ~S_SG_Ising();
		
		virtual bool			operator== (const State& state2) const;
		virtual bool			operator!= (const State& state2) const;

		virtual void			operator= (const S_SG_Ising& sg2);
		
		
			/*! Returns the negated energy of the spinglass. */
		virtual int			getFitness() const;
		
			/*! Returns the state specific energy, i.e.
			 * E = sum ( J(i,j)*spin(i)*spin(j) ) for all 0<=i<j<=spinnumber
			 * and J(i,j) = 1 iif spin i and j are neighbored and 0 otherwise.*/
		virtual double		getEnergy() const;
		
			/*! Returns the minimal number of steps via valid
			 *  neighbored states from this to another valid State.
			 *  @param state2 the State to reach
			 *  @return if state2 is a S_SG_Ising state the minimal distance or
			 *          UINT_MAX otherwise
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const;
		
			/*! Returns a pointer to a clone of the current state. */
		virtual State*			clone() const;
		
			/*! Returns a +- std::string representation of this spin glass array.
			 */
		virtual std::string	toString() const;

		// neighborhood
			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a specific successive order.
			 */
		virtual NeighborListPtr getNeighborList() const;

			/*! Returns a virtual list of all VALID neighbored states in
			 *  the energy landscape in a random order.
			 */
		virtual NeighborListPtr getRandomNeighborList() const;

			/*! Returns a VALID random neighbored state.
			 *  If a state is given it is overwritten and changed to a neighbor.
			 */
		virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const;
		
		
		
		
		//////////////// ITERABLE_NEIGHBORS ///////////////////////////////
		
		  //! How many neighbor indices are accessible.
		  //! @return the number of neighbors
		virtual size_t getNeighborNumber(void) const;
		
		  //! Access to a specific neighbor.
		  //! @param index the index of the neighbor in [0, getNeighborNumber())
		  //! @param neigh if != NULL this state should be converted into the 
		  //!              neighbor otherwise a new one is created and returned
		  //! @return the new neighbor state or NULL if the index is >= 
		  //! getNeighborNumber()
		virtual State* getNeighbor(const size_t index, State* neigh) const;
		
		  //! Undo the changes done to this state to generate the given neighbor.
		  //! The goal is to do less operations than doing a full copy like
		  //! (*neigh = *this).
		  //! @param index the neighbor index that was applied last 
		  //!              [ 0, getNeighborNumber() )
		  //! @param neigh the state resulting from the last changes ( != NULL)
		  //! @return the copy of this object via undoing the last changes in 
		  //!         neigh
		virtual State* undoNeighborChange(const size_t index, State* const neigh) const;
		
		  //! Applies the necessary changes to a given state to get the neighbor.
		  //! Note: neigh is expected to be a copy (i.e. *this == *neigh) and
		  //! therefore direct changes are possible or to be NULL. If 
		  //! (neigh==NULL) a copy of this has to be generated, modified, and
		  //! returned.
		  //! @param index the neighbor to generate
		  //! @param neigh a copy of this the changes should be applied to
		  //! @return the neighbor state of the given index or NULL if the index is
		  //!         out of range
		virtual State* applyNeighborChange(const size_t index, State* const neigh) const;

		//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
		
			/*! Access to a compressed sequence representation of the state.
			 * @return the compressed sequence representation
			 */
		virtual CSequence compress(void) const ;
		
			/*! Access to a compressed sequence representation of the state.
			 * @param toFill a data structure to write the compressed 
			 *               representation too
			 * @return the compressed sequence representation
			 */
		virtual CSequence& compress(CSequence& toFill) const;
		
			/*! Uncompresses a compressed sequencce representation into a new
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @param toFill a state object to uncompress too or NULL if a new 
			 *               object has to be created
			 * @return new State object that is encoded in cseq or NULL in error
			 *         case.
			 */
		virtual State* uncompress(const CSequence& cseq, State* toFill) const;
		
			/*! Uncompresses a compressed sequencce representation into a this
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @return this or NULL in error case
			 */
		virtual State* uncompress(const CSequence& cseq);
		
		

	protected:
	
		  //! flips the given spin (used for neighbor generation)
		inline SpinType flipSpin( SpinType s ) const {
			return ( s == SPIN_UP ? SPIN_DOWN : SPIN_UP);
		}

	}; // class
	
} // namespace ell

#endif /*S_SG_ISING_HH_*/
