#ifndef S_SG_NPP_HH_
#define S_SG_NPP_HH_

#include "ell/spinglass/S_SG_Ising.hh"
#include <vector>

namespace ell
{
	
	/*! An instance of the number partitioning problem
	 */
	class S_SG_NPP : public S_SG_Ising
	{
	public:
		typedef std::vector<double> WeightType;
		
	protected:
		
		  //! the spin ring representation
		using S_SG_Ising::spin;
		  //! the weight for each spin
		const WeightType* weights;
	public:
		  //! Creates a spin-ring of weights.size(). It is initializes with 
		  //! + spins or randomly. The weight of each spin is given by weights.
		  //! @param weights the weight of each spin --> its size determines the
		  //!               number of spins in the ring 
		  //! @param random initialize all spins randomly or with + 
		S_SG_NPP(const WeightType* weights, bool random = true);

		  //! Creates a spin-ring from the string representatio spinStr. 
		  //! The weight of each spin is given by weights.
		  //! @param weights the weight of each spin 
		  //! @param spinStr the +- string spin representation
		S_SG_NPP(const WeightType* weights, const std::string& spinStr);

		S_SG_NPP(const S_SG_NPP& s2);
		
		  //! Destruction
		virtual ~S_SG_NPP();

		virtual bool	operator== (const State& state2) const;
		virtual bool	operator!= (const State& state2) const;

		virtual void	operator= (const S_SG_NPP& sg2);

			/*! Returns the state specific energy, i.e.
			 * E = abs( sum ( weights(i)*spin(i) ) ) for all 0<=i<j<=spinnumber
			 * and spin(i)=+-1 .*/
		virtual double		getEnergy() const;
			/*! Returns a pointer to a clone of the current state. */
		virtual State*			clone() const;

			/*! Returns a VALID random neighbored state.
			 *  If a state is given it is overwritten and changed to a neighbor.
			 */
		virtual State* getRandomNeighbor(State* inPlaceNeigh = NULL) const;

		  //! Access to a specific neighbor.
		  //! @param index the index of the neighbor in [0, getNeighborNumber())
		  //! @param neigh if != NULL this state should be converted into the 
		  //!              neighbor otherwise a new one is created and returned
		  //! @return the new neighbor state or NULL if the index is >= 
		  //! getNeighborNumber()
		virtual State* getNeighbor(const size_t index, State* neigh) const;
		
		  //! Applies the necessary changes to a given state to get the neighbor.
		  //! Note: neigh is expected to be a copy (i.e. *this == *neigh) and
		  //! therefore direct changes are possible or to be NULL. If 
		  //! (neigh==NULL) a copy of this has to be generated, modified, and
		  //! returned.
		  //! @param index the neighbor to generate
		  //! @param neigh a copy of this the changes should be applied to
		  //! @return the neighbor state of the given index or NULL if the index is
		  //!         out of range
		virtual State* applyNeighborChange(const size_t index, State* const neigh) const;

			/*! Uncompresses a compressed sequencce representation into a new
			 * State object.
			 * @param cseq the compressed sequence representation of a state
			 * @param toFill a state object to uncompress too or NULL if a new 
			 *               object has to be created
			 * @return new State object that is encoded in cseq or NULL in error
			 *         case.
			 */
		virtual State* uncompress(const CSequence& cseq, State* toFill) const;
		
		
	};


} // namespace ell

#endif /*S_SG_NPP_HH_*/
