#include "ell/Walk.hh"
#include "ell/SC_Listing.hh"
#include "ell/State.hh"
#include <iostream>
#include <string>
#include <vector>
#include "biu/RandomNumberGenerator.hh"
#include "biu/RandomNumberFactory.hh"

using namespace ell;

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double energy;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _energy) 
		{
			id = _id;
			energy = _energy;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			if ( ((DummyState*)elem)->getEnergy()==0 ) return NULL;
			else return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, energy);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy) { id = _id; energy = _energy; } 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return true; }
	bool operator!= (const State& state2) const { return true; }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy); }
	std::string	toString() const { return id; }
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy));
	}
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy));
	}
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const { return NULL; }
	DummyState& applyMove() {
		if (energy > 0) {
			energy -= 1;
		}
		return *this;
	}
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
	CSequence  compress(void) const {
		return CSequence();
	}
	
	CSequence&   compress(CSequence& toFill) const {
		toFill.clear();
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq, State* toFill) const {
		return toFill;
	}
	
	State*   uncompress(const CSequence& cseq) {
		return NULL;
	}

};

int main(int argc, char **argv) {
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : walkRandom\n"
				<<"===========================================" <<std::endl;

		// init randomizer	
	std::cout	<<"=> RND::setRNG( biu::RNG_LCG() )" <<std::endl;
	biu::RandomNumberGenerator* rng = new biu::RNG_LCG();
	biu::RNF::setRNG( rng );
	delete rng;
	const unsigned int seed = 1;
	std::cout	<<"=> biu::RNF::setSeed("<<seed<<")" <<std::endl;
	biu::RNF::getRNG().setSeed(seed);
	
	std::cout	<< "=> creating DummyState" << std::endl;
	State* state = new DummyState("state 1", 10);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;

	// stop after 5 successors
	unsigned int l = 5;
	std::cout	<< "=> creating WAC_MaxLength("<<l<<")" << std::endl;
	WalkAbortionCriterion* wac = new WAC_MaxLength(l);
	std::cout	<< "=> creating StateCollector" << std::endl;
	StateCollector* sc = new SC_Listing();
	
	std::cout	<< "=> calling walk" << std::endl;
	sc = WalkRandom::walkRandom(state, sc, wac);
	
	std::cout 	<< "Number Of Accepted States (should be 6): " 
					<< sc->size()
					<< std::endl;
	
	SC_Listing* sch;
	sch = dynamic_cast<SC_Listing*>(sc);
	
	std::cout 	<< "Accepted States:" << std::endl;
	SC_Listing::List list = sch->getList();
	SC_Listing::List::iterator iter;
  	for( iter = list.begin(); iter != list.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
	
  	delete state;
  	delete wac;
  	delete sc;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
